#ifndef LSM_STORE_NODE_INTERNAL
#define LSM_STORE_NODE_INTERNAL

#include "lsm.h"

/**
 * A node inside a store node's internal binary tree.
 */
typedef struct lsm_store_inode {
  struct lsm_store_inode *left;
  struct lsm_store_inode *right;
  struct lsm_store_node *next;
  char key;
} lsm_store_inode;

/**
 * Initialize a new lsm_store_inode.
 */
lsm_error lsm_store_inode_init(lsm_store_inode **ptr, const char c);

/**
 * A node inside the store's trie structure. Internally, each node manages a
 * binary tree.
 */
typedef struct lsm_store_node {
  /* lsm_entry *entry; */
  lsm_store_inode *root;
  uint8_t size;
  /* lsm_string skip; */
} lsm_store_node;

/**
 * Initialize a new lsm_store_node.
 */
lsm_error lsm_store_node_init(lsm_store_node **out);

/**
 * Search for the next node following the given character, if present.
 */
lsm_error lsm_store_node_search(lsm_store_node **out, lsm_store_node *node,
                                const char c);

#endif
