#include <stdio.h>

#include "lander.h"
#include "log.h"

#define ENV(var, env_var)                                                      \
  const char *var = getenv(env_var);                                           \
  if (var == NULL) {                                                           \
    critical(1, "Missing environment variable %s", env_var);                   \
  }                                                                            \
  var = strdup(var);

#define ENV_OPT(var, env_var, default)                                         \
  const char *var = getenv(env_var);                                           \
  if (var == NULL) {                                                           \
    var = strdup(default);                                                     \
  } else {                                                                     \
    var = strdup(var);                                                         \
  }

int main() {
  setvbuf(stdout, NULL, _IONBF, 0);

  ENV(api_key, "LANDER_API_KEY");
  ENV_OPT(port_str, "LANDER_PORT", "18080");
  ENV_OPT(data_dir, "LANDER_DATA_DIR", ".");

  int port = atoi(port_str);

  if (port <= 0 || port >= 1 << 16) {
    critical(1, "Invalid TCP port %s", port_str);
  }

  char file_path[strlen(data_dir) + 12 + 1];
  sprintf(file_path, "%s/lander.data", data_dir);

  info("Initializing trie from file '%s'", file_path);

  Trie *trie;
  TrieExitCode res = trie_init(&trie, file_path);

  if (res != Ok) {
    critical(1, "An error occured while populating the trie.");
  }

  info("Trie initialized and populated with %i entries", trie_size(trie));

  http_loop_gctx *gctx = http_loop_gctx_init();
  gctx->trie = trie;
  gctx->routes = lander_routes;
  gctx->route_count = sizeof(lander_routes) / sizeof(lander_routes[0]);
  gctx->api_key = api_key;
  gctx->data_dir = data_dir;
  event_loop *el = http_loop_init(gctx);

  http_loop_run(el, port);
}
